searchNodes=[{"doc":"ForkWithFlags, the Elixir feature flag library. This module provides the public interface to the library and its API is made of three simple methods to enable, disable and query feature flags. In their simplest form, flags can be toggled on and off globally. More advanced rules or &quot;gates&quot; are available, and they can be set and queried for any term that implements these protocols: The ForkWithFlags.Actor protocol can be implemented for types and structs that should have specific rules. For example, in web applications it's common to use a %User{} struct or equivalent as an actor, or perhaps the current country of the request. The ForkWithFlags.Group protocol can be implemented for types and structs that should belong to groups for which one wants to enable and disable some flags. For example, one could implement the protocol for a %User{} struct to identify administrators. See the Usage notes for a more detailed explanation.","ref":"ForkWithFlags.html","title":"ForkWithFlags","type":"module"},{"doc":"Returns a list of all flag names currently configured, as atoms. This can be useful for debugging or for display purposes, but it's not meant to be used at runtime. Undefined flags, for example, will be considered disabled.","ref":"ForkWithFlags.html#all_flag_names/0","title":"ForkWithFlags.all_flag_names/0","type":"function"},{"doc":"Returns a list of all the flags currently configured, as data structures. This function is provided for debugging and to build more complex functionality (e.g. it's used in the web GUI), but it is not meant to be used at runtime to check if a flag is enabled. To query the value of a flag, please use the enabled?2 function instead.","ref":"ForkWithFlags.html#all_flags/0","title":"ForkWithFlags.all_flags/0","type":"function"},{"doc":"Clears the data of a feature flag. Clears the data for an entire feature flag or for a specific Actor or Group gate. Clearing a boolean gate is not supported because a missing boolean gate is equivalent to a disabled boolean gate. Sometimes enabling or disabling a gate is not what you want, and you need to remove that gate's rules instead. For example, if you don't need anymore to explicitly enable or disable a flag for an actor, and the default state should be used instead, you'll want to clear the gate. It's also possible to clear the entire flag, by not passing any option. Options for_actor: an_actor - used to clear the flag for a specific term only. The value can be any term that implements the Actor protocol. for_group: a_group_name - used to clear the flag for a specific group only. The value should be a binary or an atom (It's internally converted to a binary and it's stored and retrieved as a binary. Atoms are supported for retro-compatibility with versions &lt;= 0.9) boolean: true - used to clear the boolean gate. for_percentage: true - used to clear any percentage gate. Examples iex&gt; alias ForkWithFlags.TestUser , as : User iex&gt; harry = % User { id : 1 , name : &quot;Harry Potter&quot; , groups : [ &quot;wizards&quot; , &quot;gryffindor&quot; ] } iex&gt; hagrid = % User { id : 2 , name : &quot;Rubeus Hagrid&quot; , groups : [ &quot;wizards&quot; , &quot;gamekeeper&quot; ] } iex&gt; dudley = % User { id : 3 , name : &quot;Dudley Dursley&quot; , groups : [ &quot;muggles&quot; ] } iex&gt; ForkWithFlags . disable ( :wands ) iex&gt; ForkWithFlags . enable ( :wands , for_group : &quot;wizards&quot; ) iex&gt; ForkWithFlags . disable ( :wands , for_actor : hagrid ) iex&gt; iex&gt; ForkWithFlags . enabled? ( :wands ) false iex&gt; ForkWithFlags . enabled? ( :wands , for : harry ) true iex&gt; ForkWithFlags . enabled? ( :wands , for : hagrid ) false iex&gt; ForkWithFlags . enabled? ( :wands , for : dudley ) false iex&gt; iex&gt; ForkWithFlags . clear ( :wands , for_actor : hagrid ) :ok iex&gt; ForkWithFlags . enabled? ( :wands , for : hagrid ) true iex&gt; iex&gt; ForkWithFlags . clear ( :wands ) :ok iex&gt; ForkWithFlags . enabled? ( :wands ) false iex&gt; ForkWithFlags . enabled? ( :wands , for : harry ) false iex&gt; ForkWithFlags . enabled? ( :wands , for : hagrid ) false iex&gt; ForkWithFlags . enabled? ( :wands , for : dudley ) false","ref":"ForkWithFlags.html#clear/2","title":"ForkWithFlags.clear/2","type":"function"},{"doc":"Disables a feature flag. Options :for_actor - used to disable the flag for a specific term only. The value can be any term that implements the Actor protocol. :for_group - used to disable the flag for a specific group only. The value should be a binary or an atom (It's internally converted to a binary and it's stored and retrieved as a binary. Atoms are supported for retro-compatibility with versions &lt;= 0.9) :for_percentage_of - used to disable the flag for a percentage of time or actors, expressed as {:time, float} or {:actors, float} , where float is in the range 0.0 &lt; x &lt; 1.0 . Examples Disable globally iex&gt; ForkWithFlags . enable ( :random_koala_gifs ) iex&gt; ForkWithFlags . enabled? ( :random_koala_gifs ) true iex&gt; ForkWithFlags . disable ( :random_koala_gifs ) { :ok , false } iex&gt; ForkWithFlags . enabled? ( :random_koala_gifs ) false Disable for an actor iex&gt; ForkWithFlags . enable ( :spider_sense ) { :ok , true } iex&gt; villain = %{ name : &quot;Venom&quot; } iex&gt; ForkWithFlags . disable ( :spider_sense , for_actor : villain ) { :ok , false } iex&gt; ForkWithFlags . enabled? ( :spider_sense ) true iex&gt; ForkWithFlags . enabled? ( :spider_sense , for : villain ) false Disable for a group This example relies on the reference implementation used in the tests. iex&gt; alias ForkWithFlags.TestUser , as : User iex&gt; harry = % User { name : &quot;Harry Potter&quot; , groups : [ &quot;wizards&quot; , &quot;gryffindor&quot; ] } iex&gt; dudley = % User { name : &quot;Dudley Dursley&quot; , groups : [ &quot;muggles&quot; ] } iex&gt; ForkWithFlags . enable ( :hogwarts ) { :ok , true } iex&gt; ForkWithFlags . disable ( :hogwarts , for_group : &quot;muggles&quot; ) { :ok , false } iex&gt; ForkWithFlags . enabled? ( :hogwarts ) true iex&gt; ForkWithFlags . enabled? ( :hogwarts , for : harry ) true iex&gt; ForkWithFlags . enabled? ( :hogwarts , for : dudley ) false Disable for a percentage of the time iex&gt; ForkWithFlags . clear ( :random_glitch ) :ok iex&gt; ForkWithFlags . disable ( :random_glitch , for_percentage_of : { :time , 0.999999999 } ) { :ok , false } iex&gt; ForkWithFlags . enabled? ( :random_glitch ) false iex&gt; ForkWithFlags . disable ( :random_glitch , for_percentage_of : { :time , 0.000000001 } ) { :ok , false } iex&gt; ForkWithFlags . enabled? ( :random_glitch ) true Disable for a percentage of the actors iex&gt; ForkWithFlags . disable ( :new_ui , for_percentage_of : { :actors , 0.3 } ) { :ok , false }","ref":"ForkWithFlags.html#disable/2","title":"ForkWithFlags.disable/2","type":"function"},{"doc":"Enables a feature flag. Options :for_actor - used to enable the flag for a specific term only. The value can be any term that implements the Actor protocol. :for_group - used to enable the flag for a specific group only. The value should be a binary or an atom (It's internally converted to a binary and it's stored and retrieved as a binary. Atoms are supported for retro-compatibility with versions &lt;= 0.9) :for_percentage_of - used to enable the flag for a percentage of time or actors, expressed as {:time, float} or {:actors, float} , where float is in the range 0.0 &lt; x &lt; 1.0 . Examples Enable globally iex&gt; ForkWithFlags . enabled? ( :super_shrink_ray ) false iex&gt; ForkWithFlags . enable ( :super_shrink_ray ) { :ok , true } iex&gt; ForkWithFlags . enabled? ( :super_shrink_ray ) true Enable for an actor iex&gt; ForkWithFlags . disable ( :warp_drive ) { :ok , false } iex&gt; ForkWithFlags . enable ( :warp_drive , for_actor : &quot;Scotty&quot; ) { :ok , true } iex&gt; ForkWithFlags . enabled? ( :warp_drive ) false iex&gt; ForkWithFlags . enabled? ( :warp_drive , for : &quot;Scotty&quot; ) true Enable for a group This example relies on the reference implementation used in the tests. iex&gt; alias ForkWithFlags.TestUser , as : User iex&gt; marty = % User { name : &quot;Marty McFly&quot; , groups : [ &quot;students&quot; , &quot;time_travelers&quot; ] } iex&gt; doc = % User { name : &quot;Emmet Brown&quot; , groups : [ &quot;scientists&quot; , &quot;time_travelers&quot; ] } iex&gt; buford = % User { name : &quot;Buford Tannen&quot; , groups : [ &quot;gunmen&quot; , &quot;bandits&quot; ] } iex&gt; ForkWithFlags . enable ( :delorean , for_group : &quot;time_travelers&quot; ) { :ok , true } iex&gt; ForkWithFlags . enabled? ( :delorean ) false iex&gt; ForkWithFlags . enabled? ( :delorean , for : buford ) false iex&gt; ForkWithFlags . enabled? ( :delorean , for : marty ) true iex&gt; ForkWithFlags . enabled? ( :delorean , for : doc ) true Enable for a percentage of the time iex&gt; ForkWithFlags . disable ( :random_glitch ) iex&gt; ForkWithFlags . enable ( :random_glitch , for_percentage_of : { :time , 0.999999999 } ) iex&gt; ForkWithFlags . enabled? ( :random_glitch ) true iex&gt; ForkWithFlags . enable ( :random_glitch , for_percentage_of : { :time , 0.000000001 } ) iex&gt; ForkWithFlags . enabled? ( :random_glitch ) false Enable for a percentage of the actors This example is based on the fact that the actor score for the actor-flag pair marty + :new_ui is lower than 50%, and for the buford + :new_ui is higher. iex&gt; ForkWithFlags . disable ( :new_ui ) iex&gt; ForkWithFlags . enable ( :new_ui , for_percentage_of : { :actors , 0.5 } ) iex&gt; ForkWithFlags . enabled? ( :new_ui ) false iex&gt; alias ForkWithFlags.TestUser , as : User iex&gt; marty = % User { id : 42 , name : &quot;Marty McFly&quot; } iex&gt; buford = % User { id : 2 , name : &quot;Buford Tannen&quot; } iex&gt; ForkWithFlags . enabled? ( :new_ui , for : marty ) true iex&gt; ForkWithFlags . enabled? ( :new_ui , for : buford ) false","ref":"ForkWithFlags.html#enable/2","title":"ForkWithFlags.enable/2","type":"function"},{"doc":"Checks if a flag is enabled. It can be invoked with just the flag name, as an atom, to check the general status of a flag (i.e. the boolean gate). Options :for - used to provide a term for which the flag could have a specific value. The passed term should implement the Actor or Group protocol, or both. Examples This example relies on the reference implementation used in the tests. iex&gt; alias ForkWithFlags.TestUser , as : User iex&gt; harry = % User { id : 1 , name : &quot;Harry Potter&quot; , groups : [ &quot;wizards&quot; , &quot;gryffindor&quot; ] } iex&gt; ForkWithFlags . disable ( :elder_wand ) iex&gt; ForkWithFlags . enable ( :elder_wand , for_actor : harry ) iex&gt; ForkWithFlags . enabled? ( :elder_wand ) false iex&gt; ForkWithFlags . enabled? ( :elder_wand , for : harry ) true iex&gt; voldemort = % User { id : 7 , name : &quot;Tom Riddle&quot; , groups : [ &quot;wizards&quot; , &quot;slytherin&quot; ] } iex&gt; ForkWithFlags . enabled? ( :elder_wand , for : voldemort ) false iex&gt; filch = % User { id : 88 , name : &quot;Argus Filch&quot; , groups : [ &quot;staff&quot; ] } iex&gt; ForkWithFlags . enable ( :magic_wands , for_group : &quot;wizards&quot; ) iex&gt; ForkWithFlags . enabled? ( :magic_wands , for : harry ) true iex&gt; ForkWithFlags . enabled? ( :magic_wands , for : voldemort ) true iex&gt; ForkWithFlags . enabled? ( :magic_wands , for : filch ) false","ref":"ForkWithFlags.html#enabled?/2","title":"ForkWithFlags.enabled?/2","type":"function"},{"doc":"Returns a ForkWithFlags.Flag struct for the given name, or nil if no flag is found. Useful for debugging.","ref":"ForkWithFlags.html#get_flag/1","title":"ForkWithFlags.get_flag/1","type":"function"},{"doc":"","ref":"ForkWithFlags.html#t:options/0","title":"ForkWithFlags.options/0","type":"type"},{"doc":"Implement this protocol to provide actors. Actor gates allows you to enable or disable a flag for one or more entities. For example, in web applications it's common to use a %User{} struct or equivalent as an actor, or perhaps the data used to represent the current country for an HTTP request. This can be useful to showcase a work-in-progress feature to someone, to gradually rollout a functionality by country, or to dynamically disable some features in some contexts (e.g. a deploy introduces a critical error that only happens in one specific country). Actor gates take precedence over the others, both when they're enabled and when they're disabled. They can be considered as toggle overrides. In order to be used as an actor, an entity must implement the ForkWithFlags.Actor protocol. This can be implemented for custom structs or literally any other type. Examples This protocol is typically implemented for some application structure. defmodule MyApp.User do defstruct [ :id , :name ] end defimpl ForkWithFlags.Actor , for : MyApp.User do def id ( %{ id : id } ) do &quot;user: \#{ id } &quot; end end bruce = % User { id : 1 , name : &quot;Bruce&quot; } alfred = % User { id : 2 , name : &quot;Alfred&quot; } ForkWithFlags.Actor . id ( bruce ) &quot;user:1&quot; ForkWithFlags.Actor . id ( alfred ) &quot;user:2&quot; ForkWithFlags . enable ( :batmobile , for_actor : bruce ) but it can also be implemented for the builtin types: defimpl ForkWithFlags.Actor , for : Map do def id ( %{ actor_id : actor_id } ) do &quot;map: \#{ actor_id } &quot; end def id ( map ) do map |&gt; inspect ( ) |&gt; ( &amp; :crypto . hash ( :md5 , &amp;1 ) ) . ( ) |&gt; Base . encode16 |&gt; ( &amp; &quot;map: \#{ &amp;1 } &quot; ) . ( ) end end defimpl ForkWithFlags.Actor , for : BitString do def id ( str ) do &quot;string: \#{ str } &quot; end end ForkWithFlags.Actor . id ( %{ actor_id : &quot;bar&quot; } ) &quot;map:bar&quot; ForkWithFlags.Actor . id ( %{ foo : &quot;bar&quot; } ) &quot;map:E0BB5BA6873E3AC34B0B6928190C1F2B&quot; ForkWithFlags.Actor . id ( &quot;foobar&quot; ) &quot;string:foobar&quot; ForkWithFlags . disable ( :foobar , for_actor : %{ actor_id : &quot;just a map&quot; } ) ForkWithFlags . enable ( :foobar , for_actor : &quot;just a string&quot; ) Actor identifiers must be globally unique binaries. Since supporting multiple kinds of actors is a common requirement, all the examples use the common technique of namespacing the IDs: defimpl ForkWithFlags.Actor , for : MyApp.User do def id ( user ) do &quot;user: \#{ user . id } &quot; end end defimpl ForkWithFlags.Actor , for : MyApp.Country do def id ( country ) do &quot;country: \#{ country . iso3166 } &quot; end end","ref":"ForkWithFlags.Actor.html","title":"ForkWithFlags.Actor","type":"protocol"},{"doc":"Should return a globally unique binary. Example iex&gt; ForkWithFlags.Actor . id ( % ForkWithFlags.TestUser { id : 313 } ) &quot;user:313&quot;","ref":"ForkWithFlags.Actor.html#id/1","title":"ForkWithFlags.Actor.id/1","type":"function"},{"doc":"","ref":"ForkWithFlags.Actor.html#t:t/0","title":"ForkWithFlags.Actor.t/0","type":"type"},{"doc":"Represents a feature flag. This module is not meant to be used directly.","ref":"ForkWithFlags.Flag.html","title":"ForkWithFlags.Flag","type":"module"},{"doc":"","ref":"ForkWithFlags.Flag.html#t:t/0","title":"ForkWithFlags.Flag.t/0","type":"type"},{"doc":"Represents a feature flag gate, that is one of several conditions attached to a feature flag. This module is not meant to be used directly.","ref":"ForkWithFlags.Gate.html","title":"ForkWithFlags.Gate","type":"module"},{"doc":"","ref":"ForkWithFlags.Gate.html#t:t/0","title":"ForkWithFlags.Gate.t/0","type":"type"},{"doc":"Implement this protocol to provide groups. Group gates are similar to actor gates, but they apply to a category of entities rather than specific ones. They can be toggled on or off for the name of the group (as an atom) instead of a specific term. Group gates take precedence over boolean gates but are overridden by actor gates. The semantics to determine which entities belong to which groups are application specific. Entities could have an explicit list of groups they belong to, or the groups could be abstract and inferred from some other attribute. For example, an :employee group could comprise all %User{} structs with an email address matching the company domain, or an :admin group could be made of all users with %User{admin: true} . In order to be affected by a group gate, an entity should implement the ForkWithFlags.Group protocol. The protocol automatically falls back to a default Any implementation, which states that any entity belongs to no group at all. This makes it possible to safely use &quot;normal&quot; actors when querying group gates, and to implement the protocol only for structs and types for which it matters. The protocol can be implemented for custom structs or literally any other type. defmodule MyApp.User do defstruct [ :email , admin : false , groups : [ ] ] end defimpl ForkWithFlags.Group , for : MyApp.User do def in? ( %{ email : email } , :employee ) , do : Regex . match? ( ~r/@mycompany.com$/ , email ) def in? ( %{ admin : is_admin } , :admin ) , do : ! ! is_admin def in? ( %{ groups : list } , group_name ) , do : group_name in list end elisabeth = % User { email : &quot;elisabeth@mycompany.com&quot; , admin : true , groups : [ :engineering , :product ] } ForkWithFlags.Group . in? ( elisabeth , :employee ) true ForkWithFlags.Group . in? ( elisabeth , :admin ) true ForkWithFlags.Group . in? ( elisabeth , :engineering ) true ForkWithFlags.Group . in? ( elisabeth , :marketing ) false defimpl ForkWithFlags.Group , for : Map do def in? ( %{ group : group_name } , group_name ) , do : true def in? ( _ , _ ) , do : false end ForkWithFlags.Group . in? ( %{ group : :dumb_tests } , :dumb_tests ) true With the protocol implemented, actors can be used with the library functions: ForkWithFlags . disable ( :database_access ) ForkWithFlags . enable ( :database_access , for_group : :engineering )","ref":"ForkWithFlags.Group.html","title":"ForkWithFlags.Group","type":"protocol"},{"doc":"Should return a boolean. The default implementation will always return false for any argument. Example iex&gt; user = %{ name : &quot;bolo&quot; , group : &quot;staff&quot; } iex&gt; ForkWithFlags.Group . in? ( data , &quot;staff&quot; ) true iex&gt; ForkWithFlags.Group . in? ( data , &quot;superusers&quot; ) false","ref":"ForkWithFlags.Group.html#in?/2","title":"ForkWithFlags.Group.in?/2","type":"function"},{"doc":"","ref":"ForkWithFlags.Group.html#t:t/0","title":"ForkWithFlags.Group.t/0","type":"type"},{"doc":"The in-memory cache for the feature flag, backed by an ETS table. This module is not meant to be used directly, but some of its functions can be useful to debug flag state.","ref":"ForkWithFlags.Store.Cache.html","title":"ForkWithFlags.Store.Cache","type":"module"},{"doc":"Returns the contents of the cache ETS table, for inspection.","ref":"ForkWithFlags.Store.Cache.html#dump/0","title":"ForkWithFlags.Store.Cache.dump/0","type":"function"},{"doc":"Clears the cache. It will be rebuilt gradually as the public interface of the package is queried.","ref":"ForkWithFlags.Store.Cache.html#flush/0","title":"ForkWithFlags.Store.Cache.flush/0","type":"function"},{"doc":"","ref":"ForkWithFlags.Store.Cache.html#t:cached_at/0","title":"ForkWithFlags.Store.Cache.cached_at/0","type":"type"},{"doc":"","ref":"ForkWithFlags.Store.Cache.html#t:ttl/0","title":"ForkWithFlags.Store.Cache.ttl/0","type":"type"},{"doc":"A behaviour module for implementing persistence adapters. The package ships with persistence adapters for Redis and Ecto, but you can provide your own adapters by adopting this behaviour.","ref":"ForkWithFlags.Store.Persistent.html","title":"ForkWithFlags.Store.Persistent","type":"behaviour"},{"doc":"Retrieves all the names of the persisted flags.","ref":"ForkWithFlags.Store.Persistent.html#c:all_flag_names/0","title":"ForkWithFlags.Store.Persistent.all_flag_names/0","type":"callback"},{"doc":"Retrieves all the persisted flags.","ref":"ForkWithFlags.Store.Persistent.html#c:all_flags/0","title":"ForkWithFlags.Store.Persistent.all_flags/0","type":"callback"},{"doc":"Deletes an entire flag, identified by name.","ref":"ForkWithFlags.Store.Persistent.html#c:delete/1","title":"ForkWithFlags.Store.Persistent.delete/1","type":"callback"},{"doc":"Deletes a gate from a flag, identified by name.","ref":"ForkWithFlags.Store.Persistent.html#c:delete/2","title":"ForkWithFlags.Store.Persistent.delete/2","type":"callback"},{"doc":"Retrieves a flag by name.","ref":"ForkWithFlags.Store.Persistent.html#c:get/1","title":"ForkWithFlags.Store.Persistent.get/1","type":"callback"},{"doc":"Persists a gate for a flag, identified by name.","ref":"ForkWithFlags.Store.Persistent.html#c:put/2","title":"ForkWithFlags.Store.Persistent.put/2","type":"callback"},{"doc":"A persistent adapter should return either a child specification if it needs any process to be started and supervised, or nil if it does not. For example, the builtin Redis persistence adapter implements this function by delegating to Redix.child_spec/1 because it needs the Redix processes to work. On the other hand, the builtin Ecto adapter implements this function by returning nil , because the Ecto repo is provided to this package by the host application, and it's assumed that the Ecto process tree is started and supervised somewhere else. This custom worker_spec/0 function is used instead of the typical child_spec/1 function because this function can return nil if the adapter doesn't need to be supervised, whereas child_spec/1 must return a valid child spec map.","ref":"ForkWithFlags.Store.Persistent.html#c:worker_spec/0","title":"ForkWithFlags.Store.Persistent.worker_spec/0","type":"callback"},{"doc":"A Module-based supervisor . It implements Supervisor.child_spec/1 to describe the supervision tree for the :fork_with_flags application. This module is used internally by the package when the :fork_with_flags OTP application starts its own supervision tree, which is the default behavior. If that is disabled, the user's host application should use this module to start the supervision tree directly. The main purpose of this API is allow the user's host application to control when FunWithFlag's supervision tree is started. This is helpful when the package is configured to depend on some of the host application's modules, e.g. the Phoenix.PubSub process ( as documented ). More detailed instructions on how to configure this in an application are available in the readme .","ref":"ForkWithFlags.Supervisor.html","title":"ForkWithFlags.Supervisor","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"ForkWithFlags.Supervisor.html#child_spec/1","title":"ForkWithFlags.Supervisor.child_spec/1","type":"function"},{"doc":"How to start this supervisor and its tree. This function is referenced by the child_spec/1 definition for this supervisor module.","ref":"ForkWithFlags.Supervisor.html#start_link/1","title":"ForkWithFlags.Supervisor.start_link/1","type":"function"},{"doc":"","ref":"ForkWithFlags.Gate.InvalidGroupNameError.html","title":"ForkWithFlags.Gate.InvalidGroupNameError","type":"exception"},{"doc":"","ref":"ForkWithFlags.Gate.InvalidTargetError.html","title":"ForkWithFlags.Gate.InvalidTargetError","type":"exception"},{"doc":"","ref":"readme.html","title":"ForkWithFlags","type":"extras"},{"doc":"This is a temporary fork of tompave 's excellent fun_with_flags library. It seems that the original library is not actively maintained at the moment, so I've created this fork. For my case, specifically to integrate unmerged changes which add support for SQLite as a database adapter. That being said I'm also happy to review and accept other contributions to the library while the original repository remains in stasis. I in no way take credit for, or attribute myself to, this library. All that goes to tompave , whom I'm happy to hand this work back over to if and whenever the time comes. Thanks ✌️","ref":"readme.html#a-note-on-the-fork","title":"ForkWithFlags - A note on the fork","type":"extras"},{"doc":"I tried my best to keep the original naming of fun_with_flags , FunWithFlags , etc, but I was unable to do so in a way that made the compiler happy, and I've done this to publish on Hex so fork_with_flags it is! A few things you'll have to do to integrate the fork... 0. Don't change the table names To ease migration to the fork, I've intentionally not changed anything about the default migrations and table names. The default table name remains fun_with_flags_toggles . So, when it comes to replacing fun_with_flags all over the place, keep the ecto_table_name: &quot;fun_with_flags_toggles&quot; config intact - unless you're already using a custom name. 1. Switch packages in mix.exs Unlock the dependencies: mix deps.unlock fun_with_flags mix deps.unlock fun_with_flags_ui # if necessary Then update the package names and install, re-compile, etc: { :fork_with_flags , &quot;~&gt; 1.11.1&quot; } , { :fork_with_flags_ui , &quot;~&gt; 0.8.1&quot; } , # if necessary 2. Replace fun_with_flags -&gt; fork_with_flags Remember: Leave the :ecto_table_name configuration option intact unless you're using a custom table name already. You'll find fun_with_flags all around in places like included_applications: [...] , dialyzer: [...] , etc. Replace those with fun_with_flags . Also ensure your replace all covers Endpoint.subscribe(&quot;fork_with_flags_changes&quot;) to become Endpoint.subscribe(&quot;fork_with_flags_changes&quot;) . 3. Replace FunWithFlags -&gt; ForkWithFlags Usage of the library itself will also need to be replaced, from FunWithFlags to ForkWithFlags . That's about it!","ref":"readme.html#switching-from-fun_with_flags","title":"ForkWithFlags - Switching from fun_with_flags","type":"extras"},{"doc":"ForkWithFlags, a fork on the Elixir feature flag library fun_with_flags . If you're reading this on the GitHub repo , keep in mind that this readme refers to the master branch. For the latest version released on Hex, please check the readme published with the docs . ForkWithFlags is an OTP application that provides a 2-level storage to save and retrieve feature flags, an Elixir API to toggle and query them, and a web dashboard as control panel. It stores flag information in Redis or a relational DB (PostgreSQL, MySQL, or SQLite - with Ecto) for persistence and synchronization across different nodes, but it also maintains a local cache in an ETS table for fast lookups. When flags are added or toggled on a node, the other nodes are notified via PubSub and reload their local ETS caches.","ref":"readme.html#readme","title":"ForkWithFlags - README","type":"extras"},{"doc":"What's a Feature Flag Usage Gate Priority and Interactions Boolean Gate Actor Gate Group Gate Percentage of Time Gate Percentage of Actors Gate Clearing a Feature Flag's Rules Web Dashboard Origin So, caching, huh? To Do Installation Configuration Persistence Adapters Ecto Multi-tenancy PubSub Adapters Extensibility Custom Persistence Adapters Application Start Behaviour Testing Development","ref":"readme.html#content","title":"ForkWithFlags - Content","type":"extras"},{"doc":"Feature flags, or feature toggles, are boolean values associated to a name. They should be used to control whether some application feature is enabled or disabled, and they are meant to be modified at runtime while an application is running. This is usually done by the people who control the application. In their simplest form, flags can be toggled on and off globally. More advanced rules or &quot;gates&quot; allow a fine grained control over their status. For example, it's possible to toggle a flag on and off for specific entities or for groups. The goal is to have more granular and precise control over what is made available to which users, and when. A common use case, in web applications, is to enable a functionality without the need to deploy or restart the server, or to enable it only for internal users to test it before rolling it out to everyone. Another scenario is the ability to quickly disable a functionality if it's causing problems. They can also be used to implement a simple authorization system, for example to an admin area.","ref":"readme.html#what-s-a-feature-flag","title":"ForkWithFlags - What's a Feature Flag?","type":"extras"},{"doc":"ForkWithFlags has a simple API to query and toggle feature flags. Most of the time, you'll call ForkWithFlags.enabled?/2 with the name of the flag and optional arguments. Different kinds of toggle gates are supported: Boolean : globally on and off. Actors : on or off for specific structs or data. The ForkWithFlags.Actor protocol can be implemented for types and structs that should have specific rules. For example, in web applications it's common to use a %User{} struct or equivalent as an actor, or perhaps the current country of the request. Groups : on or off for structs or data that belong to a category or satisfy a condition. The ForkWithFlags.Group protocol can be implemented for types and structs that belong to groups for which a feature flag can be enabled or disabled. For example, one could implement the protocol for a %User{} struct to identify administrators. %-of-Time : globally on for a percentage of the time. It ignores actors and groups. Mutually exclusive with the %-of-actors gate. %-of-Actors : globally on for a percentage of the actors. It only applies when the flag is checked with a specific actor and is ignored when the flag is checked without actor arguments. Mutually exclusive with the %-of-time gate. Boolean, Actor and Group gates can express either an enabled or disabled state. The percentage gates can only express an enabled state, as disabling something for a percentage of time or actors is logically equivalent to enabling it for the complementary percentage. Gate Priority and Interactions The priority order is from most to least specific: Actors &gt; Groups &gt; Boolean &gt; Percentage , and it applies to both enabled and disabled gates. For example, a disabled group gate takes precedence over an enabled boolean (global) gate for the entities in the group, and a further enabled actor gate overrides the disabled group gate for a specific entity. When an entity belongs to multiple groups with conflicting toggle status, the disabled group gates have precedence over the enabled ones. The percentage gates are checked last, if present, and they're only checked if no other gate is enabled. As another example, a flag can have a disabled boolean gate and a 50% enabled %-of-actors gate. When the flag is checked with an actor, it has a (deterministic, consistent and repeatable) 50% chance to be enabled, but when checked without an actor argument it will always be disabled. If we add to the flag a disabled actor gate and an enabled group gate, the flag will be always disabled for the actor, always enabled for any other actor matching the group, have a 50% change to be enabled for any other actor, and always be disabled when checked without actor arguments. If, then, we replace the 50%-of-actors gate with a 50%-of-time gate, the flag will be always disabled for the actor, always enabled for any other actor matching the group, and have a 50% chance to be enabled for any other actor or when checked without an actor argument. Boolean Gate The boolean gate is the simplest one. It's either enabled or disabled, globally. It's also the gate with the second lowest priority (it can mask the percentage gates). If a flag is undefined, it defaults to be globally disabled. ForkWithFlags . enabled? ( :cool_new_feature ) false { :ok , true } = ForkWithFlags . enable ( :cool_new_feature ) ForkWithFlags . enabled? ( :cool_new_feature ) true { :ok , false } = ForkWithFlags . disable ( :cool_new_feature ) ForkWithFlags . enabled? ( :cool_new_feature ) false Actor Gate This allows you to enable or disable a flag for one or more entities. For example, in web applications it's common to use a %User{} struct or equivalent as an actor, or perhaps the data used to represent the current country for an HTTP request. This can be useful to showcase a work-in-progress feature to someone, to gradually rollout a functionality by country, or to dynamically disable some features in some contexts. Actor gates take precedence over the others, both when they're enabled and when they're disabled. They can be considered as toggle overrides. In order to be used as an actor, an entity must implement the ForkWithFlags.Actor protocol. This can be implemented for custom structs or literally any other type. defmodule MyApp.User do defstruct [ :id , :name ] end defimpl ForkWithFlags.Actor , for : MyApp.User do def id ( %{ id : id } ) do &quot;user: \#{ id } &quot; end end bruce = % MyApp.User { id : 1 , name : &quot;Bruce&quot; } alfred = % MyApp.User { id : 2 , name : &quot;Alfred&quot; } ForkWithFlags.Actor . id ( bruce ) &quot;user:1&quot; ForkWithFlags.Actor . id ( alfred ) &quot;user:2&quot; defimpl ForkWithFlags.Actor , for : Map do def id ( %{ actor_id : actor_id } ) do &quot;map: \#{ actor_id } &quot; end def id ( map ) do map |&gt; inspect ( ) |&gt; ( &amp; :crypto . hash ( :md5 , &amp;1 ) ) . ( ) |&gt; Base . encode16 |&gt; ( &amp; &quot;map: \#{ &amp;1 } &quot; ) . ( ) end end ForkWithFlags.Actor . id ( %{ actor_id : &quot;bar&quot; } ) &quot;map:bar&quot; ForkWithFlags.Actor . id ( %{ foo : &quot;bar&quot; } ) &quot;map:E0BB5BA6873E3AC34B0B6928190C1F2B&quot; With the protocol implemented, actors can be used with the library functions: { :ok , true } = ForkWithFlags . enable ( :restful_nights ) { :ok , false } = ForkWithFlags . disable ( :restful_nights , for_actor : bruce ) { :ok , true } = ForkWithFlags . enable ( :batmobile , for_actor : bruce ) ForkWithFlags . enabled? ( :restful_nights ) true ForkWithFlags . enabled? ( :batmobile ) false ForkWithFlags . enabled? ( :restful_nights , for : alfred ) true ForkWithFlags . enabled? ( :batmobile , for : alfred ) false ForkWithFlags . enabled? ( :restful_nights , for : bruce ) false ForkWithFlags . enabled? ( :batmobile , for : bruce ) true Actor identifiers must be globally unique binaries. Since supporting multiple kinds of actors is a common requirement, all the examples use the common technique of namespacing the IDs: defimpl ForkWithFlags.Actor , for : MyApp.User do def id ( user ) do &quot;user: \#{ user . id } &quot; end end defimpl ForkWithFlags.Actor , for : MyApp.Country do def id ( country ) do &quot;country: \#{ country . iso3166 } &quot; end end Group Gate Group gates are similar to actor gates, but they apply to a category of entities rather than specific ones. They can be toggled on or off for the name of the group instead of a specific term. Group gates take precedence over boolean gates but are overridden by actor gates. Group names can be binaries or atoms. Atoms are supported for retro-compatibility with versions &lt;= 0.9 and binaries are therefore preferred. In fact, atoms are internally converted to binaries and are then stored and later retrieved as binaries. The semantics to determine which entities belong to which groups are application specific. Entities could have an explicit list of groups they belong to, or the groups could be abstract and inferred from some other attribute. For example, an :employee group could comprise all %User{} structs with an email address matching the company domain, or an :admin group could be made of all users with %User{admin: true} . In order to be affected by a group gate, an entity should implement the ForkWithFlags.Group protocol. The protocol automatically falls back to a default Any implementation, which states that any entity belongs to no group at all. This makes it possible to safely use &quot;normal&quot; actors when querying group gates, and to implement the protocol only for structs and types for which it matters. The protocol can be implemented for custom structs or literally any other type. defmodule MyApp.User do defstruct [ :email , admin : false , groups : [ ] ] end defimpl ForkWithFlags.Group , for : MyApp.User do def in? ( %{ email : email } , &quot;employee&quot; ) , do : Regex . match? ( ~r/@mycompany.com$/ , email ) def in? ( %{ admin : is_admin } , &quot;admin&quot; ) , do : ! ! is_admin def in? ( %{ groups : list } , group_name ) , do : group_name in list end elisabeth = % MyApp.User { email : &quot;elisabeth@mycompany.com&quot; , admin : true , groups : [ &quot;engineering&quot; , &quot;product&quot; ] } ForkWithFlags.Group . in? ( elisabeth , &quot;employee&quot; ) true ForkWithFlags.Group . in? ( elisabeth , &quot;admin&quot; ) true ForkWithFlags.Group . in? ( elisabeth , &quot;engineering&quot; ) true ForkWithFlags.Group . in? ( elisabeth , &quot;marketing&quot; ) false defimpl ForkWithFlags.Group , for : Map do def in? ( %{ group : group_name } , group_name ) , do : true def in? ( _ , _ ) , do : false end ForkWithFlags.Group . in? ( %{ group : &quot;dumb_tests&quot; } , &quot;dumb_tests&quot; ) true With the protocol implemented, actors can be used with the library functions: ForkWithFlags . disable ( :database_access ) ForkWithFlags . enable ( :database_access , for_group : &quot;engineering&quot; ) ForkWithFlags . enabled? ( :database_access ) false ForkWithFlags . enabled? ( :database_access , for : elisabeth ) true Percentage of Time Gate %-of-time gates are similar to boolean gates, but they allow to enable a flag for a percentage of the time. In practical terms, this means that a percentage of the enabled?() calls for a flag will return true, regardless of the presence of an actor argument. When a %-of-time gate is checked a pseudo-random number is generated and compared with the percentage value of the gate. If the result of the random roll is lower than the gate's percentage value, the gate is considered enabled. So, at the risk of stating the obvious and for the sake of clarity, a 90% gate is enabled more often than a 10% gate. %-of-time gates are useful to gradually introduce alternative code paths that either have the same effects of the old ones, or don't have effects visible to the users. This last point is important, because with a %-of-time gate the application will behave differently on a pseudo-random basis. A good use case for %-of-time gates is to safely test the correctness or performance and load characteristics of an alternative implementation of a functionality. For example: ForkWithFlags . clear ( :alternative_implementation ) ForkWithFlags . enable ( :alternative_implementation , for_percentage_of : { :time , 0.05 } ) def foo ( bar ) do if ForkWithFlags . enabled? ( :alternative_implementation ) do new_foo ( bar ) else old_foo ( bar ) end end The %-of-time gate is incompatible and mutually exclusive with the %-of-actors gate, and it replaces it when it gets set. While there are ways to make them work together, it would needlessly overcomplicate the priority rules. Percentage of Actors Gate %-of-actors gates are similar to the %-of-time gates, but instead of using a pseudo-random chance they calculate the actor scores using a deterministic, consistent and repeatable function that factors in the flag name. At a high level: actor |&gt; ForkWithFlags.Actor . id ( ) |&gt; sha256_hash ( flag_name ) |&gt; hash_to_percentage ( ) Since the scores depend on both the actor ID and the flag name, they're guaranteed to always be the same for each actor-flag combination. At the same time, the same actor will have different scores for different flags, and each flag will have a uniform distribution of scores for all the actors. Just like for the %-of-time gates, an actor's score is compared with the gate's percentage value and, if lower, the gate will result enabled. A practical example, based on the ForkWithFlags.Actor protocol set up from the previous sections: defmodule MyApp.User do defstruct [ :id , :name ] end defimpl ForkWithFlags.Actor , for : MyApp.User do def id ( %{ id : id } ) do &quot;user: \#{ id } &quot; end end frodo = % MyApp.User { id : 1 , name : &quot;Frodo Baggins&quot; } sam = % MyApp.User { id : 2 , name : &quot;Samwise Gamgee&quot; } pippin = % MyApp.User { id : 3 , name : &quot;Peregrin Took&quot; } merry = % MyApp.User { id : 4 , name : &quot;Meriadoc Brandybuck&quot; } ForkWithFlags.Actor.Percentage . score ( frodo , :pipeweed ) 0.8658294677734375 ForkWithFlags.Actor.Percentage . score ( sam , :pipeweed ) 0.68426513671875 ForkWithFlags.Actor.Percentage . score ( pippin , :pipeweed ) 0.510528564453125 ForkWithFlags.Actor.Percentage . score ( merry , :pipeweed ) 0.2617645263671875 { :ok , true } = ForkWithFlags . enable ( :pipeweed , for_percentage_of : { :actors , 0.60 } ) ForkWithFlags . enabled? ( :pipeweed , for : frodo ) false ForkWithFlags . enabled? ( :pipeweed , for : sam ) false ForkWithFlags . enabled? ( :pipeweed , for : pippin ) true ForkWithFlags . enabled? ( :pipeweed , for : merry ) true { :ok , true } = ForkWithFlags . enable ( :pipeweed , for_percentage_of : { :actors , 0.685 } ) ForkWithFlags . enabled? ( :pipeweed , for : sam ) true ForkWithFlags.Actor.Percentage . score ( pippin , :pipeweed ) 0.510528564453125 ForkWithFlags.Actor.Percentage . score ( pippin , :mushrooms ) 0.6050872802734375 ForkWithFlags.Actor.Percentage . score ( pippin , :palantir ) 0.144073486328125 Once a %-of-actors gate has been defined for a flag, the same actor will always see the same result (unless its actor or group gates are set, or the flag gets globally enabled). Also, this means that as long the percentage value of the gate will increase and never decrease, actors for which the gate has been enabled will always see it enabled. This is ideal to gradually roll out new functionality to users. For example, in a Phoenix application: ForkWithFlags . clear ( :new_design ) ForkWithFlags . enable ( :new_design , for_percentage_of : { :actors , 0.2 } ) ForkWithFlags . enable ( :new_design , for_group : &quot;beta_testers&quot; ) defmodule MyPhoenixApp.MyView do use MyPhoenixApp , :view def render ( &quot;my_template.html&quot; , assigns ) do if ForkWithFlags . enabled? ( :new_design , for : assigns . user ) do render ( &quot;new_template.html&quot; , assigns ) else render ( &quot;old_template.html&quot; , assigns ) end end end The %-of-actors gate is incompatible and mutually exclusive with the %-of-time gate, and it replaces it when it gets set. While there are ways to make them work together, it would needlessly overcomplicate the priority rules. Clearing a Feature Flag's Rules Sometimes enabling or disabling a gate is not what you want, and removing that gate's rules would be more correct. For example, if you don't need anymore to explicitly enable or disable a flag for an actor or for a group, and the default state should be used instead, clearing the gate is the right choice. More examples: alias ForkWithFlags.TestUser , as : User harry = % User { id : 1 , name : &quot;Harry Potter&quot; , groups : [ &quot;wizards&quot; , &quot;gryffindor&quot; ] } hagrid = % User { id : 2 , name : &quot;Rubeus Hagrid&quot; , groups : [ &quot;wizards&quot; , &quot;gamekeeper&quot; ] } dudley = % User { id : 3 , name : &quot;Dudley Dursley&quot; , groups : [ &quot;muggles&quot; ] } ForkWithFlags . disable ( :wands ) ForkWithFlags . enable ( :wands , for_group : &quot;wizards&quot; ) ForkWithFlags . disable ( :wands , for_actor : hagrid ) ForkWithFlags . enabled? ( :wands ) false ForkWithFlags . enabled? ( :wands , for : harry ) true ForkWithFlags . enabled? ( :wands , for : hagrid ) false ForkWithFlags . enabled? ( :wands , for : dudley ) false ForkWithFlags . clear ( :wands , for_actor : hagrid ) ForkWithFlags . enabled? ( :wands , for : hagrid ) true ForkWithFlags . clear ( :wands , for_group : &quot;wizards&quot; ) ForkWithFlags . enabled? ( :wands , for : hagrid ) false ForkWithFlags . enabled? ( :wands , for : harry ) false ForkWithFlags . enable ( :magic_powers , for_percentage_of : { :time , 0.0001 } ) ForkWithFlags . clear ( :magic_powers , for_percentage : true ) For completeness, clearing the boolean gate is also supported. ForkWithFlags . enable ( :wands ) ForkWithFlags . enabled? ( :wands ) true ForkWithFlags . enabled? ( :wands , for : harry ) true ForkWithFlags . enabled? ( :wands , for : hagrid ) false ForkWithFlags . enabled? ( :wands , for : dudley ) true ForkWithFlags . clear ( :wands , boolean : true ) ForkWithFlags . enabled? ( :wands ) false ForkWithFlags . enabled? ( :wands , for : harry ) true ForkWithFlags . enabled? ( :wands , for : hagrid ) false ForkWithFlags . enabled? ( :wands , for : dudley ) false It's also possible to clear an entire flag. ForkWithFlags . clear ( :wands ) ForkWithFlags . enabled? ( :wands ) false ForkWithFlags . enabled? ( :wands , for : harry ) false ForkWithFlags . enabled? ( :wands , for : hagrid ) false ForkWithFlags . enabled? ( :wands , for : dudley ) false","ref":"readme.html#usage","title":"ForkWithFlags - Usage","type":"extras"},{"doc":"An optional extension of this library is ForkWithFlags.UI , a web graphical control panel. It's a Plug, so it can be embedded in a host Phoenix or Plug application or served standalone.","ref":"readme.html#web-dashboard","title":"ForkWithFlags - Web Dashboard","type":"extras"},{"doc":"This library is heavily inspired by the flipper Ruby gem . Having used Flipper in production at scale, this project aims to improve in two main areas: Minimize the load on the persistence layer: feature flags are not toggled that often, and there is no need to query Redis or the DB for each check. Be more reliable: it should keep working with the latest cached values even if Redis becomes unavailable, although with the risk of nodes getting out of sync. (if the DB becomes unavailable, feature flags are probably the last of your problems) Just as Elixir and Phoenix are meant to scale better than Ruby on Rails with high levels of traffic and concurrency, ForkWithFlags should aim to be more scalable and reliable than Flipper.","ref":"readme.html#origin","title":"ForkWithFlags - Origin","type":"extras"},{"doc":"There are only two hard things in Computer Science: cache invalidation and naming things. -- Phil Karlton The reason to add an ETS cache is that, most of the time, feature flags can be considered static values. Doing a round-trip to the DB (Redis, PostgreSQL or MySQL) is expensive in terms of time and in terms of resources, especially if multiple flags must be checked during a single web request. In the worst cases, the load on the DB can become a cause of concern, a performance bottleneck or the source of a system failure. Often the solution is to memoize the flag values in the context of the web request , but the approach can be extended to the scope of the entire server. This is what ForkWithFlags does, as each application node/instance caches the flags in an ETS table. Of course, caching adds a different kind of complexity and there are some pros and cons. When a flag is created or updated the ETS cache on the local node is updated immediately, and the main problem is synchronizing the flag data across the other application nodes that should share the same view of the world. For example, if we have two or more nodes running the application, and on one of them an admin user updates a flag that the others have already cached, or creates a flag that the others have already looked up (and cached as &quot;disabled&quot;), then the other nodes must be notified of the changes. ForkWithFlags uses three mechanisms to deal with the problem: Use PubSub to emit change notifications. All nodes subscribe to the same channel and reload flags in the ETS cache when required. If that fails, the cache has a configurable TTL. Reading from the DB every few minutes is still better than doing so 30k times per second. If that doesn't work, it's possible to disable the cache and just read from the DB all the time. That's what Flipper does. In terms of performance, very synthetic benchmarks (where the DBs run on the same machine as the Beam code, so with no network hop but sharing the CPU) show that the ETS cache makes querying the ForkWithFlags interface between 10 and 20 times faster than going directly to Redis, and between 20 and 40 times faster than going directly to Postgres. The variance depends on the complexity of the flag data to be retrieved.","ref":"readme.html#so-caching-huh","title":"ForkWithFlags - So, caching, huh?","type":"extras"},{"doc":"Add some optional randomness to the TTL, so that Redis or the DB don't get hammered at constant intervals after a server restart.","ref":"readme.html#to-do","title":"ForkWithFlags - To Do","type":"extras"},{"doc":"The package can be installed by adding fork_with_flags to your list of dependencies in mix.exs . In order to have a small installation footprint, the dependencies for the different adapters are all optional. You must explicitly require the ones you wish to use. def deps do [ { :fork_with_flags , &quot;~&gt; 1.11&quot; } , # either: { :redix , &quot;~&gt; 0.9&quot; } , # or: { :ecto_sql , &quot;~&gt; 3.0&quot; } , # optionally, if you don&#39;t want to use Redis&#39; builtin pubsub { :phoenix_pubsub , &quot;~&gt; 2.0&quot; } , ] end Using ecto_sql for persisting the flags also requires an ecto adapter, e.g. postgrex , mariaex or myxql . Please refer to the Ecto documentation for the details. Since ForkWithFlags depends on an Elixir more recent than 1.4, there is no need to explicitly declare the application . If you need to customize how the :fork_with_flags application is loaded and started, refer to the Application Start Behaviour section, below in this document.","ref":"readme.html#installation","title":"ForkWithFlags - Installation","type":"extras"},{"doc":"The library can be configured in host applications through Mix and the config.exs file. This example shows some default values: config :fork_with_flags , :cache , enabled : true , ttl : 900 # in seconds # the Redis persistence adapter is the default, no need to set this. config :fork_with_flags , :persistence , [ adapter : ForkWithFlags.Store.Persistent.Redis ] # this can be disabled if you are running on a single node and don&#39;t need to # sync different ETS caches. It won&#39;t have any effect if the cache is disabled. # The Redis PuSub adapter is the default, no need to set this. config :fork_with_flags , :cache_bust_notifications , [ enabled : true , adapter : ForkWithFlags.Notifications.Redis ] # Notifications can also be disabled, which will also remove the Redis/Redix dependency config :fork_with_flags , :cache_bust_notifications , [ enabled : false ] When using Redis for persistence and/or cache-busting PubSub it is necessary to configure the connection to the Redis instance. These options can be omitted if Redis is not being used. For example, the defaults: # the Redis options will be forwarded to Redix. config :fork_with_flags , :redis , host : &quot;localhost&quot; , port : 6379 , database : 0 # a URL string can be used instead config :fork_with_flags , :redis , &quot;redis://localhost:6379/0&quot; # or a {URL, [opts]} tuple config :fork_with_flags , :redis , { &quot;redis://localhost:6379/0&quot; , socket_opts : [ :inet6 ] } # a {:system, name} tuple can be used to read from the environment config :fork_with_flags , :redis , { :system , &quot;REDIS_URL&quot; } Redis Sentinel is also supported. See the Redix docs for more details. config :fork_with_flags , :redis , sentinel : [ sentinels : [ &quot;redis:://locahost:1234/1&quot; ] , group : &quot;primary&quot; , ] , database : 5 Persistence Adapters The library comes with persistence adapters for the Redix and Ecto libraries, that allow to persist feature flag data in Redis, PostgreSQL, MySQL, or SQLite. In order to use any of them, you must declare the correct optional dependency in the Mixfile (see the installation instructions, above). The Redis adapter is the default and there is no need to explicitly declare it. All it needs is the Redis connection configuration. In order to use the Ecto adapter, an Ecto repo must be provided in the configuration. ForkWithFlags expects the Ecto repo to be initialized by the host application, which also needs to start and supervise any required processes. If using Phoenix this is managed automatically by the framework, and it's fine to use the same repo used by the rest of the application. Only PostgreSQL (via postgrex ), MySQL (via mariaex or myxql ), and SQLite (via ecto_sqlite3 ) are supported at the moment. Support for other RDBMSs might come in the future. To configure the Ecto adapter: # Normal Phoenix and Ecto configuration. # The repo can either use the Postgres or MySQL adapter. config :my_app , ecto_repos : [ MyApp.Repo ] config :my_app , MyApp.Repo , username : &quot;my_db_user&quot; , password : &quot;my secret db password&quot; , database : &quot;my_app_dev&quot; , hostname : &quot;localhost&quot; , pool_size : 10 # ForkWithFlags configuration. config :fork_with_flags , :persistence , adapter : ForkWithFlags.Store.Persistent.Ecto , repo : MyApp.Repo , ecto_table_name : &quot;your_table_name&quot; # optional # Default table name is &quot;fun_with_flags_toggles&quot;. It's also necessary to create the DB table that will hold the feature flag data. To do that, create a new migration in your project and copy the contents of the provided migration file . Then run the migration . When using the Ecto persistence adapter, ForkWithFlags will annotate all queries using the Ecto Repo Query API with a custom option: [fun_with_flags: true] . This is done to make it easier to identify ForkWithFlags queries when working with Ecto customization hooks, e.g. the Ecto.Repo.prepare_query/3 callback . Since this sort of annotations via custom query options are only useful with the Ecto Query API ( context ), other repo functions are not annotated with the custom option. Ecto Multi-tenancy If you followed the Ecto guide on setting up multi-tenancy with foreign keys , you must add an exception for queries originating from ForkWithFlags. As mentioned in the section above, these queries have a custom query option named :fork_with_flags set to true : # Sample code, only relevant if you followed the Ecto guide on multi tenancy with foreign keys. defmodule MyApp.Repo do use Ecto.Repo , otp_app : :my_app require Ecto.Query @impl true def prepare_query ( _operation , query , opts ) do cond do # add the check for opts[:fork_with_flags] here: opts [ :skip_org_id ] || opts [ :schema_migration ] || opts [ :fork_with_flags ] -&gt; { query , opts } org_id = opts [ :org_id ] -&gt; { Ecto.Query . where ( query , org_id : ^ org_id ) , opts } true -&gt; raise &quot;expected org_id or skip_org_id to be set&quot; end end end PubSub Adapters The library comes with two PubSub adapters for the Redix and Phoenix.PubSub libraries. In order to use any of them, you must declare the correct optional dependency in the Mixfile. (see the installation instructions, below) The Redis PubSub adapter is the default and doesn't need to be explicitly configured. It can only be used in conjunction with the Redis persistence adapter however, and is not available when using Ecto for persistence. When used, it connects directly to the Redis instance used for persisting the flag data. The Phoenix PubSub adapter uses the high level API of Phoenix.PubSub , which means that under the hood it could use either its PG2 or Redis adapters, and this library doesn't need to know. It's provided as a convenient way to leverage distributed Erlang when using ForkWithFlags in a Phoenix application, although it can be used independently (without the rest of the Phoenix framework) to add PubSub to Elixir apps running on Erlang clusters. ForkWithFlags expects the Phoenix.PubSub process to be started by the host application, and in order to use this adapter the client (name or PID) must be provided in the configuration. For example, in Phoenix (&gt;= 1.5.0) it would be: # normal Phoenix configuration config :my_app , MyApp.Web.Endpoint , pubsub_server : MyApp.PubSub # ForkWithFlags configuration config :fork_with_flags , :cache_bust_notifications , enabled : true , adapter : ForkWithFlags.Notifications.PhoenixPubSub , client : MyApp.PubSub Or, without Phoenix: # possibly in the application&#39;s supervision tree children = [ { Phoenix.PubSub , [ name : :my_pubsub_process_name , adapter : Phoenix.PubSub.PG2 ] } ] opts = [ strategy : :one_for_one , name : MyApp.Supervisor ] { :ok , _pid } = Supervisor . start_link ( children , opts ) # config/config.exs config :fork_with_flags , :cache_bust_notifications , enabled : true , adapter : ForkWithFlags.Notifications.PhoenixPubSub , client : :my_pubsub_process_name","ref":"readme.html#configuration","title":"ForkWithFlags - Configuration","type":"extras"},{"doc":"Custom Persistence Adapters This library aims to be extensible and allows users to provide their own persistence layer. This is supported through ForkWithFlags.Store.Persistent , a generic persistence behaviour that is adopted by the builtin Redis and Ecto adapters. Custom persistence adapters can adopt the behaviour and then be configured as the persistence module in the Mix config of the user applications. For example, an application can define this module: defmodule MyApp.MyAlternativeFlagStore do @behaviour ForkWithFlags.Store.Persistent # implement all the behaviour&#39;s callback end And then configure the library to use it: config :fork_with_flags , :persistence , adapter : MyApp.MyAlternativeFlagStore","ref":"readme.html#extensibility","title":"ForkWithFlags - Extensibility","type":"extras"},{"doc":"As explained in the Installation section, above in this document, the :fork_with_flags application will start automatically when you add the package as a dependency in your Mixfile. The :fork_with_flags application starts its own supervision tree which manages all required processes and is provided by the ForkWithFlags.Supervisor module. Sometimes, this can cause issues and race conditions if ForkWithFlags is configured to rely on Erlang processes that are owned by another application. For example, if you have configured the Phoenix.PubSub cache-busting notification adapter, one of FunWithFlag's processes will immediately try to subscribe to its notifications channel using the provided PubSub process identifier. If that process is not available, ForkWithFlags will retry a few times and then give up and raise an exception. This will become a problem if you're using ForkWithFlags in a large application (e.g. a Phoenix app) and the :fork_with_flags application starts much faster than the Phoenix supervision tree. In these cases, it's better to directly control how ForkWithFlags starts its processes. The first step is to add the ForkWithFlags.Supervisor module directly to the supervision tree of the host application. For example, in a Phoenix app it would look like this: defmodule MyPhoenixApp.Application do @moduledoc false use Application def start(_type, _args) do children = [ MyPhoenixApp.Repo, MyPhoenixAppWeb.Telemetry, {Phoenix.PubSub, name: MyPhoenixApp.PubSub}, MyPhoenixAppWeb.Endpoint, + ForkWithFlags.Supervisor, ] opts = [strategy: :one_for_one, name: MyPhoenixApp.Supervisor] Supervisor.start_link(children, opts) end # ... Then it's necessary to configure the Mix project to not start the :fork_with_flags application automatically. This can be accomplished in the Mixfile in a number of ways, for example: ( Note : These are alternative solutions, you don't need to do both. You must decide which is more appropriate for your setup.) Option A : Declare the :fork_with_flags dependency with either the runtime: false or app: false options. ( docs ) - {:fork_with_flags, &quot;~&gt; 1.6&quot;}, + {:fork_with_flags, &quot;~&gt; 1.6&quot;, runtime: false}, If you use releases then you'll also need to modify the releases section in mix.exs so that it loads the fork_with_flags application explicitly (since runtime: false / app: false will exclude it from the assembled release). def project do [ app: :my_phoenix_app, + releases: [ + my_phoenix_app: [ + applications: [ + fork_with_flags: :load + ] + ] ] end Option B : Declare that the :fork_with_flags application is managed directly by your host application ( docs ). def application do [ mod: {MyPhoenixApp.Application, []}, + included_applications: [:fork_with_flags], extra_applications: [:logger, :runtime_tools] ] end The result of those changes is that the :fork_with_flags application won't be loaded and started automatically, and therefore the ForkWithFlags supervision tree won't risk to be started before the other processes in the host Phoenix application. Rather, the supervision tree will start alongside the other core Phoenix processes. One final note on this topic is that if you're also using ForkWithFlags.UI (refer to the Web Dashboard section, above in this document), then that will need to be configured as well. The reason is that :fork_with_flags is a dependency of :fork_with_flags_ui , so including the latter as a dependency will cause the former to be auto-started despite the configuration described above. To avoid this, the same configuration should be used for the :fork_with_flags_ui dependency, regardless of the approach used (Option A: runtime: false , app: false ; or Option B: included_applications ).","ref":"readme.html#application-start-behaviour","title":"ForkWithFlags - Application Start Behaviour","type":"extras"},{"doc":"This library depends on Redis, PostgreSQL and MySQL, and you'll need them installed and running on your system in order to run the complete test suite. The tests will use the Redis db number 5 and then clean after themselves, but it's safer to start Redis in a directory where there is no dump.rdb file you care about to avoid issues. The Ecto tests will use the SQL sandbox and all transactions will be automatically rolled back. To setup the test DB for the Ecto persistence tests, run: MIX_ENV = test PERSISTENCE = ecto mix do ecto . create , ecto . migrate # for postgres rm - rf _build / test / lib / fork_with_flags / MIX_ENV = test PERSISTENCE = ecto RDBMS = mysql mix do ecto . create , ecto . migrate # for mysql rm - rf _build / test / lib / fork_with_flags / MIX_ENV = test PERSISTENCE = ecto RDBMS = sqlite mix do ecto . create , ecto . migrate # for sqlite Then, to run all the tests: $ mix test.all The test.all task will run the test suite multiple times with different configurations to exercise a matrix of options and adapters. The Mixfile defines a few other helper tasks that allow to run the test suite with some more specific configurations.","ref":"readme.html#testing","title":"ForkWithFlags - Testing","type":"extras"},{"doc":"Like for testing, developing ForkWithFlags requires local installations of Redis, PostgreSQL and MySQL. For work that doesn't touch the persistence adapters too closely, it's possibly simpler to just run ForkWithFlags with Redis and then let CI run the tests with the other adapters. A common workflow is to run the tests and interact with the package API in iex . With the default configuration , iex -S mix will compile and load ForkWithFlags with Redis persistence and Redis PubSub. To compile and run the package in iex with Ecto and Phoenix PubSub support instead, use these commands: bin/console_ecto postgres bin/console_ecto mysql This package uses the credo and dialyxir ( dialyzer ) packages to help with local development. Their mix tasks can be executed in the root directory of the project: mix credo mix dialyzer","ref":"readme.html#development","title":"ForkWithFlags - Development","type":"extras"},{"doc":"","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"Mistake in README.","ref":"changelog.html#v1-11-2","title":"Changelog - v1.11.2","type":"extras"},{"doc":"README updates. Update broadcast channel to &quot;fork_with_flags_changes&quot; .","ref":"changelog.html#v1-11-1","title":"Changelog - v1.11.1","type":"extras"},{"doc":"Add support for SQLite with the ecto_sqlite3 adapter.","ref":"changelog.html#v1-11-0","title":"Changelog - v1.11.0","type":"extras"},{"doc":"Redis notifications adapter: minor internal changes to what data is passed to the supervisor in the child spec. ( pull/148 ) Redis notifications adapter: fix an issue that would arise when configuring both a Redis URL string and key-word options (this config API was introduced in v1.10.0): the config would work for the Redis persistence adapter, but not for the Redis notifications adapter. (Thanks iamvery , pull/149 )","ref":"changelog.html#v1-10-1","title":"Changelog - v1.10.1","type":"extras"},{"doc":"Add support for Elixir 1.14. Drop support for Elixir 1.11. Elixir &gt;= 1.12 is now required. Dropping support for older versions of Elixir simply means that this package is no longer tested with them in CI, and that compatibility issues are not considered bugs. Drop support for Erlang/OTP 22, and Erlang/OTP &gt;= 23 is now required. Dropping support for older versions of Erlang/OTP simply means that this package is not tested with them in CI, and that no compatibility issues are considered bugs. Ecto persistence adapter: ForkWithFlags will now pass a custom option when using the Ecto Repo Query API : [fun_with_flags: true] . This is done to make it easier to identify ForkWithFlags queries when working with Ecto customization hooks, e.g. the Ecto.Repo.prepare_query/3 callback . (Thanks SteffenDE , pull/143 ) Redis persistence adapter: added support to configure Redis with a {&quot;redis URL&quot;, [...kw opts]} tuple, as is supported in Redix itself . (Thanks iamvery , pull/145 )","ref":"changelog.html#v1-10-0","title":"Changelog - v1.10.0","type":"extras"},{"doc":"Drop support for Elixir 1.10. Elixir &gt;= 1.11 is now required. Dropping support for older versions of Elixir simply means that this package is no longer tested with them in CI, and that compatibility issues are not considered bugs. Relax supported versions of postgrex to allow ~&gt; 0.16 . Use Application.compile_env/3 to read the persistence config at compile time, which is used to configure the DB table name when using the Ecto persistence adapter (among other things). This fixes another instance of the issue where users of the package would change the config after compilation and observe unexpected inconsistencies and errors. ( pull/130 ) Redis adapters: add support to configure Redis Sentinel . Please see the Redix docs for more details. (Thanks parkdoyeon , pull/107 .) More precise conditional checks when deciding whether Ecto files should be compiled. ( pull/140 ) Improved documentation for running the package in a custom supervision tree, when using releases. (Thanks zaid , pull/139 )","ref":"changelog.html#v1-9-0","title":"Changelog - v1.9.0","type":"extras"},{"doc":"Lock postgrex dependency to &lt; 0.16 . Version 0.16 requires Elixir 1.11 ( changelog ) and it doesn't compile with Elixit 1.10, which ForkWithFlags still supports.","ref":"changelog.html#v1-8-1","title":"Changelog - v1.8.1","type":"extras"},{"doc":"Add support for Elixir 1.13. Drop support for Elixir 1.9. Elixir &gt;= 1.10 is now required. Dropping support for older versions of Elixir simply means that this package is no longer tested with them in CI, and that compatibility issues are not considered bugs. Removed all uses of defdelegate/2 . They caused some references to configured modules (that can change according to the config) to be reified at compile time, which lead to unexpected behaviour. They've been replaced with plain old function definitions that do the same job. (Thanks connorlay , pull/111 .) Local dev: Update the config for the library to use Config instead of the deprecated Mix.Config . For the avoidance of doubt: this has no effect when using the package in your projects, because the config/*.exs files are not present in the bundles downloaded from Hex.pm. Use Application.compile_env/3 to read the cache configuration at compile-time, which is used to define a module attribute (therefore, set at compile-time). That part of the config is compiled into a module attribute for performance reasons, and it has been a long standing issue because users of the package would get confused by their config changes not being reflected in an already compiled application ( link to relevant section in previous version of the readme ). Now, if the relevant configuration changes, users will get a clear error . Improve error handling in different layers of the package. From the persistence adapters all the way to the public functions of the top-level module. In practice, this means that some situations that would have caused a MatchError now instead will bubble up an error tuple. Most importantly, this does not affect the signature or behaviour of the ForkWithFlags.enabled?/2 function, which continues to return a simple boolean. ( pull/120 ) Typespec improvements. These include new typespecs for previously unspecced functions, amended typespecs for the new error tuples that are now bubbled up (see previous point) and fixed typespec that incorrectly ignored a returned error tuple. ( pull/120 ) The typespecs for the ForkWithFlags.Store.Persistence Elixir behaviour have been updated (see previous point). Users of the package who implemented their own custom persistence adapters are encouraged to double-check that these respect the typespecs. ( pull/120 )","ref":"changelog.html#v1-8-0","title":"Changelog - v1.8.0","type":"extras"},{"doc":"Add support for Elixir 1.12. Drop support for Elixir 1.8. Elixir &gt;= 1.9 is now required. Dropping support for older versions of Elixir simply means that this package is no longer tested with them in CI, and that compatibility issues are not considered bugs. Drop support for Erlang/OTP 21, and Erlang/OTP &gt;= 22 is now required. Dropping support for older versions of Erlang/OTP simply means that this package is not tested with them in CI, and that no compatibility issues are considered bugs. Added support for the Erlang dialyzer (via the dialyxir package). Addressed all dialyzer warnings. Fixed some incorrect typespecs and simplified the implementation of some functions. Miscellaneous documentation fixes and improvements. (Thanks kianmeng , pull/89 , pull/90 and pull/112 .) Documented the ForkWithFlags.Store.Cache module, and its Cache.flush/0 and Cache.dump/0 functions. They're now part of the public API of the package. Introduced a new ForkWithFlags.Supervisor module to manage the supervision tree for the package. The supervision strategy and configuration are unchanged, and host applications don't need to do anything to upgrade. However, this module is part of the public API of the package and can be used to better control the start behaviour of ForkWithFlags. This has also been documented in a new section of the readme . Internal changes to stop using an undocumented feature of Elixir that will go away in future versions. This affects how the function to calculate Actor scores for the %-of-actors gate is invoked, but that's an internal change, so it won't affect users of the package unless they're using undocumented features. (Thanks kelvinst , pull/105 .)","ref":"changelog.html#v1-7-0","title":"Changelog - v1.7.0","type":"extras"},{"doc":"Add support for Elixir 1.11. Drop support for Elixir 1.7. Elixir &gt;= 1.8 is now required. Dropping support for older versions of Elixir simply means that this package is no longer tested with them in CI, and that compatibility issues are not considered bugs. More internal changes to not compile in the package configuration. Removed compile-time references to the Ecto repo and the Ecto table name. See the release notes for v1.5.1 (below) for more details on this type of changes. Ecto and Postgres persistence: when updating percentage gates, use a flag-scoped advisory lock rather than locking the entire table. With the old system the entire table was locked when setting or changing any percentage gate, across all flags. With this change, the lock is scoped to one flag and the table is never fully locked. Dev and test fixes to support Phoenix.PubSub on OTP 23 and Elixir &gt;= 1.10.3. This was only an issue when working locally, and there should be no problems when using the previous version of the package in a host application. Update Redix to 1.0. As its changelog says this doesn't introduce breaking changes, but it's a major version bump that should be documented here, as it will require changes in the host applications mix files.","ref":"changelog.html#v1-6-0","title":"Changelog - v1.6.0","type":"extras"},{"doc":"Internal changes to not compile in the persistence adapter config. This has no effect on the functionality of the package, but now the Ecto or Redis adapter configuration is not memoized anymore, and it can be changed with no need to recompile the package.","ref":"changelog.html#v1-5-1","title":"Changelog - v1.5.1","type":"extras"},{"doc":"Drop support for Elixir 1.6. Elixir &gt;= 1.7 is now required. Drop support for Erlang/OTP 20, and Erlang/OTP &gt;= 21 is now required. An older Erlang/OTP might still work with older versions of Elixir, but Elixir 1.10 requires Erlang/OTP &gt;= 21. Dropping support for older versions of Erlang/OTP simply means that this package is not tested with them in CI, and that no compatibility issues are considered bugs. Upgrade Phoenix.PubSub dependency to 2.0. This provides compatibility with Phoenix 1.5. Typespec improvements. (Thanks skylerparr , pull/57 ) Internal changes to how flag data is cached in the ETS table. This has no effect on the functionality of the package, with two exceptions. First, the cache: [ttl: seconds] config value is not memoized anymore and it can be changed without recompiling. Second, since the TTL is now stored with the ETS entries, old and new ETS data is not compatible; this is not an issue if you restart/rotate your application nodes/instances when deploying, but it will be an issue if you perform hot code upgrades . In that case, you have to first empty the ETS table, for example with ForkWithFlags.Store.Cache.flush/0 . New config option to set a custom name for the DB table when using the Ecto persistence adapter. (Thanks BobbyMcWho , pull/64 and pull/77 )","ref":"changelog.html#v1-5-0","title":"Changelog - v1.5.0","type":"extras"},{"doc":"Typespec improvements. (Thanks LostKobrakai , pull/49 ) Improve Redis error handling for connection and command errors. (Thanks chubarovNick , pull/54 )","ref":"changelog.html#v1-4-1","title":"Changelog - v1.4.1","type":"extras"},{"doc":"This release focuses on making it easier to extend the package, for example with custom persistence adapters. Define a behaviour in the ForkWithFlags.Store.Persistence module that can be implemented by custom persistence adapters. The builtin Redis and Ecto adapters now formally implement this new behaviour. Refactor how cache-busting change notifications are published: move the logic out of the two builtin persistence adapters and into the level above them. While this is just an internal change, it narrows the responsibilities of the persistence adapters and simplifies implementing custom ones. Update the supervision tree to use Elixir v1.5 style child specs . Print a helpful error if a project is configured to use a persistence adapter without including its dependency packages. This mirrors what happens when the dependencies for a notifications adapter are missing. Document the Flag and Gate types, previously private. Redis persistence: relax Redix version lock to ~&gt; 0.9 , which allows to use Redix 0.10 . It was previously locked to ~&gt; 0.9.1 because of breaking changes in the last few Redix minor version releases, but going forward if it happens again it can be handled with a patch level release on ForkWithFlags.","ref":"changelog.html#v1-4-0","title":"Changelog - v1.4.0","type":"extras"},{"doc":"Ecto persistence: added support for MySQL, via either mariaex or myxql . While both are working today, the test suite uses myxql because ecto_sql is going to deprecate the mariaex adapter in the future. (Thanks stewart for starting this work with pull/41 and for helping out on pull/42 ).","ref":"changelog.html#v1-3-0","title":"Changelog - v1.3.0","type":"extras"},{"doc":"Fix invalid typespec that was causing compiler warnings on Elixir 1.8. (Thanks asummers , pull/34 )","ref":"changelog.html#v1-2-1","title":"Changelog - v1.2.1","type":"extras"},{"doc":"Redis persistence: upgrade to Redix 0.9, which deprecates Redix.PubSub. The pubsub capabilities are now part of the base Redix package. This means that ForkWithFlags also needs to drop the dependency on Redix.PubSub. Compatibility updates in the tests for Elixir 1.8. There is no other change in this release, but this is a minor version bump because upgrading Redix and dropping Redix.PubSub will require applications to also update their dependencies.","ref":"changelog.html#v1-2-0","title":"Changelog - v1.2.0","type":"extras"},{"doc":"Drop support for Elixir 1.5. Elixir &gt;= 1.6 is now required. Drop support for OTP 19. OTP &gt;= 20 is now required. Update to Ecto 3 with the ecto_sql package. Update to Redix 0.8 and Redix.PubSub 0.5. Ecto persistence: add NOT NULL constraints to the table definition in the Ecto migration. This is not a breaking change: the constraints have been added because those values are never null anyway. If users of the library want to add them, they can do so by adding this migration to their projects. Redis persistence: allow to configure the Redis URL with a system tuple to read it from an environment variable. (Thanks seangeo , pull/29 )","ref":"changelog.html#v1-1-0","title":"Changelog - v1.1.0","type":"extras"},{"doc":"This release introduces the last two gates that were initially planned and marks a milestone for the project. The API is now stable, and the project can graduate to 1.0.0 . This release doesn't introduce any breaking change, however, and users of the library should be able to upgrade without problems. If you're also using ForkWithFlags.UI then make sure to also upgrade that to version 0.4.0 , which adds GUI support for the new features. New Gates: Percentage of time gate Percentage of actors gate","ref":"changelog.html#v1-0-0","title":"Changelog - v1.0.0","type":"extras"},{"doc":"Add ability to clear the boolean gate only (useful for debugging). Added ForkWithFlags.get_flag/1 , to retrieve a flag struct. Useful for debugging. Internal improvements.","ref":"changelog.html#v0-11-0","title":"Changelog - v0.11.0","type":"extras"},{"doc":"Improvements: Ecto persistence: explicitly set the table primary key as an integer type. This improves the compatibility with Ecto repos where primary keys default to a binary type, e.g. UUID. (Thanks coryodaniel , pull/23 )","ref":"changelog.html#v0-10-1","title":"Changelog - v0.10.1","type":"extras"},{"doc":"Possibly Breaking Changes: Allow binaries and atoms as group gate names. Binaries are now preferred (atom group names are internally converted, stored and retrieved as binaries) and atoms are still allowed for retro-compatibility. While calling ForkWithFlags.enable(:foo, for_group: :bar) is still allowed and continues to work as before, this change will impact implementations of the ForkWithFlags.Group protocol that assume that the group name is passed as an atom. To safely upgrade, these implementations should be changed to work with the group names passed as a binary instead. See the update to the protocol implementation used in the tests for an example. Other changes: Compatibility updates for Ecto 2.2 (dev env, was fine in prod)","ref":"changelog.html#v0-10-0","title":"Changelog - v0.10.0","type":"extras"},{"doc":"Bug Fixes: Fixed another issue with modules referencing Ecto.","ref":"changelog.html#v0-9-2","title":"Changelog - v0.9.2","type":"extras"},{"doc":"Bug Fixes: Fixed an issue with module referencing Ecto that was not wrapped in a Code.ensure_loaded? block, which prevented the library from being used in projects that did not include Ecto.","ref":"changelog.html#v0-9-1","title":"Changelog - v0.9.1","type":"extras"},{"doc":"Ecto persistence adapter. It's now possible to store flag data with Ecto instead of Redis; if used in conjunction with the Phoenix.PubSub adapter, it's possible to use this library in Phoenix without Redis. The redix dependency is now optional. Added optional ecto dependency.","ref":"changelog.html#v0-9-0","title":"Changelog - v0.9.0","type":"extras"},{"doc":"Mark the redix_pubsub dependency as optional. Clearer error reporting for missing adapter dependencies.","ref":"changelog.html#v0-8-1","title":"Changelog - v0.8.1","type":"extras"},{"doc":"New Features: Added support for Phoenix.PubSub as an alternative transport for the cache busting notifications. Added ability to enable the ETS cache but disable the cache-busting notifications, as it can be useful when running on a single node. Other changes: Upgraded redix and redix_pubsub dependencies. Internal project and supervision changes to better support different adapters. Updated the Mix configuration options. More rational test setup.","ref":"changelog.html#v0-8-0","title":"Changelog - v0.8.0","type":"extras"},{"doc":"Bug fixes: Resolved an issue with the PubSub connection process that would crash the entire supervision tree in case of abrupt disconnection from Redis. Ops!","ref":"changelog.html#v0-7-1","title":"Changelog - v0.7.1","type":"extras"},{"doc":"New Features: ForkWithFlags.all_flags/0 is now public and documented. Added ForkWithFlags.all_flag_names/0 , public and documented. Added proper log statements via the Elixir Logger . Setting the log level to debug will print cache busting info, for example. Internal changes: Updated the redix and redix_pubsub dependencies. Extracted the private persistence and notifications modules into a redis-specific namespace. Added a config option to customize the adapters on startup, and an internal API that allows to develop other adapters to use alternative persistence and notification layers. The persistence adapters are responsible for declaring their own notifications module (if any). The provided default ones keep using Redis, and they work in tandem. At the moment, no official support for other adapters is planned, redix stays as a dependency.","ref":"changelog.html#v0-7-0","title":"Changelog - v0.7.0","type":"extras"},{"doc":"New features: Added ForkWithFlags.clear/2 to delete a specific gate or an entire flag. This is useful if you don't need an actor or group override and want to use the default boolean rule instead. Clearing a flag or a gate uses the same PubSub cache busting functionality used when updating a flag. Added ForkWithFlags.all_flags/0 , to return a list of all the flags stored in Redis. Undocumented because it's meant to build a GUI.","ref":"changelog.html#v0-6-0","title":"Changelog - v0.6.0","type":"extras"},{"doc":"Added the Group protocol and group gates. It's now possible to enable or disable a flag for a group name and implement Group for types and structs that should belong to groups.","ref":"changelog.html#v0-5-0","title":"Changelog - v0.5.0","type":"extras"},{"doc":"Added the Actor protocol and actor gates. It's now possible to implement Actor for some type or struct and then enable or disable a flag for some specific values.","ref":"changelog.html#v0-4-0","title":"Changelog - v0.4.0","type":"extras"},{"doc":"Always raise exceptions if Redis becomes unavailable and there is no cached value (expired or not). This means that both with or without cache, failures to load a flag's data will never be silently converted to the flag being disabled.","ref":"changelog.html#v0-3-0","title":"Changelog - v0.3.0","type":"extras"},{"doc":"Significant internal rewrite: use structures instead of raw booleans. Given the scope of the changes and the fact that this is still a 0.x release, bump the version number.","ref":"changelog.html#v0-2-0","title":"Changelog - v0.2.0","type":"extras"},{"doc":"Enhancements Treat cache misses and expired cached values differently. If Redis becomes unavailable, and an expired value is available in the cache, use the expired value even though normally it would be discarded.","ref":"changelog.html#v0-1-1","title":"Changelog - v0.1.1","type":"extras"},{"doc":"First usable release with the a stable initial feature set. Simple boolean flags Elixir API to enable, disable and query the flags Supervision tree, embeddable in host applications Persistence in Redis ETS cache Cache busting with TTLs Cache busting with Redis PubSub Resistant to Redis connection issues if the values are cached Option to disable the ETS cache (Redis-only mode)","ref":"changelog.html#v0-1-0","title":"Changelog - v0.1.0","type":"extras"},{"doc":"Unstable releases.","ref":"changelog.html#v0-0-x","title":"Changelog - v0.0.x","type":"extras"}]